/**
 * webpack 配置 - rule
 */

const MiniCssExtractPlugin = require('mini-css-extract-plugin')
const { getMode, isWithSourceMap, isWithHash } = require('../common/util')

module.exports = ({ config }) => {
  const mode = getMode()
  const sourceMap = isWithSourceMap()
  const hash = isWithHash()

  // rule - js
  const babelLoaderOptions = {
    presets: [
      [
        '@babel/preset-env', {
          targets: { 'browsers': ['android >= 4'] }
        }
      ]
    ],
    plugins: [
      'babel-plugin-transform-vue-jsx',
      ['@babel/plugin-proposal-decorators', { legacy: true }],
      ['@babel/plugin-proposal-class-properties', { loose: true }],
      '@babel/plugin-proposal-optional-chaining',
      '@babel/plugin-proposal-nullish-coalescing-operator',
      ['@babel/plugin-proposal-pipeline-operator', { proposal: 'minimal' }],
      '@babel/plugin-syntax-dynamic-import',
      ['@babel/plugin-transform-runtime', { corejs: 2 }]
    ]
  }

  config
    .module
    .rule('js')
    .test(/\.*(js|jsx)$/)
    .exclude
    .add(/node_modules/)
    .end()
    .use('babel')
    .loader('babel-loader')
    .options(babelLoaderOptions)
    .end()

  // rule - vue
  config
    .module
    .rule('vue')
    .test(/\.vue$/)
    .exclude
    .add(/node_modules/)
    .end()
    .use('vue')
    .loader('vue-loader')
    .options({
      compilerOptions: {
        preserveWhitespace: false
      }
    })
    .end()

  // rule - html
  config
    .module
    .rule('html')
    .test(/\.html$/)
    .exclude
    .add(/node_modules/)
    .end()
    .use('html')
    .loader('html-loader')
    .end()

  // rule - css & scss
  const cssLoaderOptions = {
    sourceMap
  }
  const postcssLoaderOptions = {
    sourceMap,
    plugins: () => [
      require('postcss-preset-env')({
        stage: 0,
        browsers: ['> 0.01%']
      })
    ]
  }
  const sassLoaderOptions = {
    sourceMap
  }
  const vueStyleLoaderOptions = {
    sourceMap
  }

  // vue css
  config
    .module
    .rule('css')
    .test(/\.css$/)
    .exclude
    .add(/node_modules/)
    .end()
    .oneOf('vue')
    .resourceQuery(/^\?vue/)
    .use('vue-style')
    .loader('vue-style-loader')
    .options(vueStyleLoaderOptions)
    .end()
    .use('css')
    .loader('css-loader')
    .options(cssLoaderOptions)
    .end()
    .use('postcss')
    .loader('postcss-loader')
    .options(postcssLoaderOptions)
    .end()
    .end()

  // 普通 css 类型文件
  config
    .module
    .rule('css')
    .test(/\.css$/)
    .exclude
    .add(/node_modules/)
    .end()
    .oneOf('normalCss')
    .use('miniCss')
    .loader(MiniCssExtractPlugin.loader)
    .end()
    .use('css')
    .loader('css-loader')
    .options(cssLoaderOptions)
    .end()
    .use('postcss')
    .loader('postcss-loader')
    .options(postcssLoaderOptions)
    .end()
    .end()

  // vue sass
  config
    .module
    .rule('sass')
    .test(/\.s(a|c)ss$/)
    .exclude
    .add(/node_modules/)
    .end()
    .oneOf('vue')
    .resourceQuery(/^\?vue/)
    .use('vue-style')
    .loader('vue-style-loader')
    .options(vueStyleLoaderOptions)
    .end()
    .use('css')
    .loader('css-loader')
    .options(cssLoaderOptions)
    .end()
    .use('postcss')
    .loader('postcss-loader')
    .options(postcssLoaderOptions)
    .end()
    .use('sass')
    .loader('sass-loader')
    .options(sassLoaderOptions)
    .end()
    .end()

  // 普通 sass 类型文件
  config
    .module
    .rule('sass')
    .test(/\.s(a|c)ss$/)
    .exclude
    .add(/node_modules/)
    .end()
    .oneOf('normalScss')
    .use('miniCss')
    .loader(MiniCssExtractPlugin.loader)
    .end()
    .use('css')
    .loader('css-loader')
    .options(cssLoaderOptions)
    .end()
    .use('postcss')
    .loader('postcss-loader')
    .options(postcssLoaderOptions)
    .end()
    .use('sass')
    .loader('sass-loader')
    .options(sassLoaderOptions)
    .end()
    .end()

  // rule - image
  config
    .module
    .rule('image')
    .test(/\.(png|jpg|gif|jpeg|svg)$/)
    .exclude
    .add(/node_modules/)
    .end()
    .use('url')
    .loader('url-loader')
    .options({
      limit: 1024 * 3,
      name: `img/[name]${hash}.[ext]`
    })
    .end()
    .when(mode === 'production',
      config => config.use('imageCompress').loader('image-webpack-loader').options({
        mozjpeg: {
          progressive: true,
          quality: 80
        },
        optipng: {
          enabled: false
        },
        pngquant: {
          quality: 80
        },
        gifsicle: {
          interlaced: false
        }
      }).end()
    )

  // rule - assets
  config
    .module
    .rule('assets')
    .test(/\.(ttf|woff|otf|eot|svga)$/)
    .exclude
    .add(/node_modules/)
    .end()
    .use('url')
    .loader('url-loader')
    .options({
      limit: 1024 * 3,
      name: `assets/[name]${hash}.[ext]`
    })
    .end()

  // rule - other
  config
    .module
    .rule('other')
    .test(/\.*(js|jsx)$/)
    .include
    .add(/ansi-regex/)
    .add(/strip-ansi/)
    .end()
    .use('babel')
    .loader('babel-loader')
    .options(babelLoaderOptions)
    .end()
}
